$TITLE  DICE version 1994 -- NLP Decomposition 

$if not set tc $set tc 80
$if not set t  $set t  40

scalar kterm /0/;

set     tc      /1*%tc%/,
        t(tc)   /1*%t%/;

$include dicedata

alias (t,tp);

PARAMETER
        dref(tc)        Reference values of damage
        eref(tc)        Reference values of emissions
        pdref(tc)       Reference shadow price on damages
        xi(t)           Post-terminal damages
        grad(tc,t)      Local dependence of D(tp) on E(t);

xi(t) = 0;
dref(t) = 0;
grad(tc,t) = 0;
eref(tc) = 0;

VARIABLES
        C(t)            Consumption trillion US dollars,
        K(t)            Capital stock trillion US dollars,
        I(t)            Investment trillion US dollars,
        Y(t)            Output net abatement and damage costs,
        D(tc)           Damage,
        A(t)            Abatement cost,
        Q(t)            Gross Output,
        E(t)            CO2-equiv emissions billion t,
        MIU(t)          Emission control rate GHGs,

        UTILITY        	Maximand;

POSITIVE VARIABLES E, Y, C, K, I, MIU;

EQUATIONS
        UTIL            Objective function
        CC(t)           Consumption
        YY(t)           Output
        AA(t)           Abatement
        QQ(t)           Underlying production function
        KK(t)           Capital balance
        DD(t)           Damage
        EE(t)           Emissions process
        TERMCAP        	Terminal capital stock constraint;

CC(t)..        	C(t) =E= Y(t) - I(t);
YY(t)..        	Y(t) =E= Q(t) - A(t)*Q(t) - D(t)*Y(t);
AA(t)..        	A(t) =E= b1 * MIU(t)**b2;
QQ(t)..        	Q(t) =E= al(t) * L(t)**(1-gamma) * K(t)**gamma;
KK(t)..        	K(t) =L= (1-dk)**10 * K(t-1) + 10 * I(t-1) + (k0*0.9)$tfirst(t);
EE(t)..        	E(t) =G= 10 * sigma(t) * (1-MIU(t)) * Q(t);
DD(t)..        	D(t) =E= dref(t) + SUM(tp, grad(t,tp)/1e6*(E(tp)-eref(tp)));
TERMCAP..       kterm =e= sum(tlast, (1-dk)**10 * K(tlast) + 10 * I(tlast));
UTIL..          UTILITY =E= SUM(t, 10 *rr(t)*L(t)*LOG(C(t)/L(t))/0.55)
                           - SUM(t, E(t) * xi(t));

*       Assign a starting point which is in the domain of the functions:

K.L(t) = k0*0.9 * L(t)/sum(tfirst,L(tfirst));
Q.L(t) = al(t) * L(t)**(1-gamma) * K.L(t)**gamma;
I.L(t) = (K.L(t+1) - (1-dk)**10*K.L(t)) / 10;
MIU.l(t) = 0.1;
E.L(t) = 10 * sigma(t) * 0.9 * Q.L(t);
A.L(t) =  b1 * MIU.L(t)**b2;
D.L(t) = 0;
Y.L(t) = Q.L(t)*(1-A.L(t))/(1 + D.L(t));
C.l(t) = Y.L(t) - I.L(t);
UTILITY.L = 1;

kterm = sum(tlast(t), K.L(tlast) * Y.L(t)/Y.L(t-1));

MIU.UP(t) = 0.99;
MIU.LO(t) = 0.01;
K.LO(t) = 0.1;
C.LO(t) = 0.1;

option nlp=conopt;

model CO2 /all/;
solve CO2 maximizing UTILITY using NLP;

eref(t) = E.L(t);
pdref(t) = -DD.M(t);
LOOP((tlast,tc)$(not t(tc)),
  eref(tc)  = eref(tlast)  * (sigma(tc)*L(tc))/(sigma(tlast)*L(tlast));
  pdref(tc) = pdref(tlast) * (L(tc)*rr(tc))   /(L(tlast)*rr(tlast));
);

PARAMETERS
        m(tc)           CO2-equiv concentration billion t,
        forc(tc)        Radiative forcing - W per m2,
        te(tc)          Temperature - atmosphere C,
        tl(tc)          Temperature - lower ocean C,
        deltaE          Difference iterval /0.01/;

m(tfirst) = M0;
te(tfirst) = T0;
tl(tfirst) = TL0;

*       Generate GAMS code for the climate model:

$onecho >climatemodel.gms
$onuni
loop(tc,
        m(tc)    = 590 + atret*eref(tc) + (1-deltam)*(m(tc-1)-590) + m0$tfirst(tc);
        forc(tc) = 4.1*(LOG(m(tc)/590)/LOG(2)) + forcoth(tc);
        te(tc) = te(tc-1)+c1*(forc(tc-1)-lam*te(tc-1)-c3*(te(tc-1)-tl(tc-1))) + t0$tfirst(tc);
        tl(tc) = tl(tc-1)+c4*(te(tc-1)-tl(tc-1)) + tl0$tfirst(tc);
        dref(tc) = (a1/9) * sqr(te(tc)););
$offecho

*       Generate GAMS code for numerical differencing

$onecho >jacobian.gms
        eref(t) = E.L(t);
        pdref(t) = -DD.M(t);
        LOOP((tlast,tc)$(not t(tc)),
          eref(tc)  = eref(tlast)  * (sigma(tc)*L(tc))/(sigma(tlast)*L(tlast));
          pdref(tc) = pdref(tlast) * (L(tc)*rr(tc))   /(L(tlast)*rr(tlast));
        );
$include climatemodel
        D.L(tc) = dref(tc);
        grad(tc,tp) = 0;
        loop(tp, eref(tp) = eref(tp) + deltaE;
$include climatemodel
          grad(tc,tp) = (dref(tc)-D.L(tc))*1e6 / deltaE;
          eref(tp) = eref(tp) - deltaE;);
        dref(tc) = D.L(tc);
        loop(tlast,
          xi(t) = sum(tc$(not t(tc)), grad(tc,t)/1e6*pdref(tc));
        );
$offecho

$include climatemodel
solve CO2 maximizing UTILITY using NLP;

parameter       itrlog(t,*)     Iteration log -- Emission control rate GHGs;
itrlog(t,"iter0") = MIU.L(t);
set iters /iter1*iter6/;
loop(iters,
$include jacobian
        solve CO2 maximizing UTILITY using NLP;
        itrlog(t,iters) = MIU.L(t);

*       Update terminal capital stock:

        kterm = sum(tlast(t), K.L(tlast) * Y.L(t)/Y.L(t-1));
);
execute_load 'mcp40.gdx',MIU;
itrlog(t,"mcp") = MIU.L(t);
$libinclude plot itrlog
